;fdftgdm_probability.pro
;
;procedure for generating FD probability distribution, i.e. calculating probability of a certain FD range.
;The distribution is based on the discrete shifted geometric distribution.
;
;The procedure needs two arrays as an input: input_data and parameters.
;
;INPUT_DATA
;5x1 array of input variables:
;column 1: v - speed of CME in km/s, float, range v > 106 km/s
;column 2: r - source position distance from the center of the solar disc in Rsun, float, range 0 ≤ r ≤ 1
;column 3: w - width of a CME, integer, range 0deg < w ≤ 360deg
;column 4: f - flare soft X-ray peak intensity in units 10−7Wm−2, float, range f > 5.3
;column 5: i - interaction level, integer, possible values: 1(none), 2 (low), 3 (medium), 4 (high)
;If a parameter is unknown, a value -1 should be given instead (outside limitations of all variables).
;If a value for the variable is given, which is outside the limitations, the procedure proceeds calculation without the variable and sends out a warning.
;
;parameters.txt
;=5x2 array of distribution parameters:
;column 1: parameter a for the y=ax+b linear function which calculates the mean of the distribution
;column 2: parameter b for the y=ax+b linear function which calculates the mean of the distribution
;
;The output is a 6x4 array of probabilities (OUTPUT_DATA):
;Columns represent separate distributions:
;column 1: probability based on speed
;column 2: probability based on source
;column 3: probability based on width
;column 4: probability based on flare
;column 5: probability based on interaction
;column 6: combined probability
;Rows represent separate Dst bins (i.e. distribution bins k, k=0,1,2,3):
;row 1: < 1% (k=0)
;row 2: 1-3% (k=1)
;row 3: 3-6% (k=2)
;row 4: > 6% (k=3)
;
;*****************************************************************************************************************************************************************
;
;CALLS UPPON:
;linear.pro (function)
;log.pro (function)
;gd_probability.pro (function)
;make_distribution.pro (procedure)
;
;*****************************************************************************************************************************************************************
;
;Function "linear" calculates linear function based on the input variable and 2 parameters:
FUNCTION linear, input, p_1, p_2
         RETURN, p_1*input+p_2
END

;Function "log" calculates logarhitmic function based on the input variable and 2 parameters:
FUNCTION log, input, p_1, p_2
         RETURN, p_1*ALOG(input)+p_2
END

;Function "gd_probability" calculates the probability of succes in each trial for the geometric probability distribution,
; using mean of the distribution as an input:
FUNCTION gd_probability, mean
         RETURN, 1/(1+mean)
END

;The procedure "make_distribution" calculates adjusted distribution based on the shifted geometric distribution (GD).
;Input:
;prob = scalar denoting the probability of succes in each trial for the GD.
;Output:
;GD_out = 1x4 array of probabilities for k=0,1,2,3
;
PRO make_distribution, prob, GD_out
    s=float(0)
    GD_out=make_array(1,4,/float)
    for k=0,3,1 do begin
          GD_out(k)=prob*(1-prob)^k
          s=s+GD_out(k)
    endfor
    for k=0,3,1 do begin
      GD_out(k)=GD_out(k)/s
    endfor
END

;main
pro fdftgdm_probability, input_data, params, output_data, output_text

;define variables
  
  output_text=make_array(7,1, /string)
  output_data=make_array(6,4, /float)
  output=make_array(1,4, /float)
  set=make_array(5,1, /string)

;MAKE DISTRIBUTIONS

  ;for speed
  if ((input_data(0) GE 106) and (input_data(0) LE 5000)) then begin 
      m=linear(input_data(0), params(0,0), params(0,1))
      p=gd_probability(m)
      make_distribution, p, output
      for k=0,3,1 do begin
          output_data(0,k)=output(k)
      endfor
      output_text(0,0)="Probability distribution for CME speed."
      set(0)='P_v,'
  endif else begin
      output_text(0,0)="No probability distribution for CME speed: input is missing."
      set(0)=''
      output=fltarr(4)
      for k=0,3,1 do begin
          output_data(0,k)=output(k)
      endfor    
  endelse
   
      
  ;for source position
  if (input_data(1) GE 0) and (input_data(1) LE 1) then begin
      m=linear(input_data(1), params(1,0), params(1,1))
      p=gd_probability(m)
      make_distribution, p, output
      for k=0,3,1 do begin
          output_data(1,k)=output(k)
      endfor
      output_text(1,0)="Probability distribution for CME/flare source position radius."
      set(1)='P_r,'
  endif else begin
      output_text(1,0)="No probability distribution for CME/flare source position radius: input is missing."
      set(1)=''
      output=fltarr(4)
      for k=0,3,1 do begin
          output_data(1,k)=output(k)
      endfor
  endelse
  
  ;for width
  if (input_data(2) GT 0) and (input_data(2) LE 360) then begin
      m=linear(input_data(2), params(2,0), params(2,1))
      p=gd_probability(m)
      make_distribution, p, output
      for k=0,3,1 do begin
          output_data(2,k)=output(k)
      endfor
      output_text(2,0)="Probability distribution for CME width."
      set(2)='P_w,'
  endif else begin
      output_text(2,0)="No probability distribution for CME width: input is missing."
      set(2)=''
      output=fltarr(4)
      for k=0,3,1 do begin
          output_data(2,k)=output(k)
      endfor
  endelse
  
  ;for flare
  if (input_data(3) GT 5.3) then begin
      m=log(input_data(3), params(3,0), params(3,1))
      p=gd_probability(m)
      make_distribution, p, output
      for k=0,3,1 do begin
          output_data(3,k)=output(k)
      endfor
      output_text(3,0)="Probability distribution for flare Soft X-ray intensity."
      set(3)='P_f,'
  endif else begin
      output_text(3,0)="No probability distribution for flare Soft X-ray intensity: input is missing."
      set(3)=''
      output=fltarr(4)
      for k=0,3,1 do begin
          output_data(3,k)=output(k)
      endfor
  endelse
  
  ;for interaction level
  if (input_data(4) EQ 1) or (input_data(4) EQ 2) or (input_data(4) EQ 3) or (input_data(4) EQ 4) then begin
      m=linear(input_data(4), params(4,0), params(4,1))
      p=gd_probability(m)
      make_distribution, p, output
      for k=0,3,1 do begin
          output_data(4,k)=output(k)
      endfor
      output_text(4,0)="Probability distribution for interaction level."
      set(4)='P_i'
  endif else begin
      output_text(4,0)="No probability distribution for interaction level: input is missing."
      set(4)=''
      output=fltarr(4)
      for k=0,3,1 do begin
          output_data(4,k)=output(k)
      endfor
  endelse


  ;combined probability distribution
  s=float(0)
  output=output_data
  for n=0,3,1 do begin
      sum_0=float(0)
      sum_1=float(0)
      sum_2=float(0)
      sum_3=float(0)
      sum_4=float(0)
      for k=0,4,1 do begin
          sum_0=sum_0+output(k,n)
          for i=1,4,1 do begin
              if (i GT k) then begin
                  prod_1=output(k,n)*output(i,n)
                  sum_1=sum_1+prod_1
              endif
              for j=2,4,1 do begin
                  if (j GT i) AND (i GT k) then begin
                      prod_2=prod_1*output(j,n)
                      sum_2=sum_2+prod_2
                  endif
                  for l=3,4,1 do begin
                      if (l GT j) AND (j GT i) AND (i GT k) then begin
                          prod_3=prod_2*output(l,n)
                          sum_3=sum_3+prod_3
                      endif
                      for p=4,4,1 do begin
                          if (p GT l) AND (l GT j) AND (j GT i) AND (i GT k) then begin
                              prod_4=prod_3*output(p,n)
                              sum_4=sum_4+prod_4
                          endif
                      endfor;p
                  endfor;l
               endfor;j
           endfor;i
       endfor;k
       output(5,n)=sum_0-sum_1+sum_2-sum_3+sum_4
       s=s+output(5,n)
   endfor;n
   for k=0,3,1 do begin
          output_data(5,k)=output(5,k)/s
   endfor

output_text(5,0)='Combined probability distribution calculated based on ' + set(0)+set(1)+set(2)+set(3)+set(4)

end



