;HVAR.pro
;
;=interactive routine to setup input for the following models:
;dbm.pro - model for heliospheric propagation of CMEs
;cgeft.pro - statistical empirical model to forecast CME geo-effectiveness
;fdft.pro - statistical empirical model to forecast CME GCR-effectiveness
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;the output of each model is found in their corresponding folder
;_________________________________________________________________________________
;DBM OUTPUT:
;is found in *\HVAR\DBM\OUTPUT
;consists of a .txt file listing:
; - warnings
; - input details
; - output: CME transit time, impact speed and arrival time at target as output details
;_________________________________________________________________________________
;CGeFT OUTPUT:
;is found in *\HVAR\CGeFT\OUTPUT
;consists of several files:
;(1) cgeft_distributions.txt
;     = 6x4 array of distributions:
;     column 1: probability based on speed
;     column 2: probability based on source
;     column 3: probability based on width
;     column 4: probability based on flare
;     column 5: probability based on interaction
;     column 6: combined probability
;     Rows represent separate Dst bins:
;     row 1: less than 100nT
;     row 2: 100-200nT
;     row 3: 200-300 nT
;     row 4: more than 300 nT
;(2) cgeft_table_text.txt
;     = 4x6 array of text for each of the coulmn of cgeft_distributions.txt
;(3) cgeft_text.txt
;     = 1x2 array of online text:
;     row 1: expected |Dst| level
;     row 2: corresponding image name
;(4) cgeft.png
;     = image of the combined probability distribution and expected |Dst| level
;_________________________________________________________________________________
;FDFT OUTPUT:
;is found in *\HVAR\FDFT\OUTPUT
;consists of several files:
;(1) fdft_distributions.txt
;     = 6x4 array of distributions:
;     column 1: probability based on speed
;     column 2: probability based on source
;     column 3: probability based on width
;     column 4: probability based on flare
;     column 5: probability based on interaction
;     column 6: combined probability
;     Rows represent separate FD bins:
;     row 1: less than 1%
;     row 2: 1-3%
;     row 3: 3-6%
;     row 4: more than 6%
;(2) fdft_table_text.txt
;     = 4x6 array of text for each of the coulmn of fdft_distributions.txt
;(3) cgeft_text.txt
;     = 1x2 array of online text:
;     row 1: expected FD level
;     row 2: corresponding image name
;(4) fdft.png
;     = image of the combined probability distribution and expected FD level


pro hvar

;define file path
path=file_dirname(routine_filepath("hvar"))

print, "Enter CME date and time (format: dd/mm/yyyy hh:mm)"
date=''
read,date
date=strcompress(date, /remove_all)
if ((strmid(date,2,1) ne "/") or (strmid(date,5,1) ne "/")) then begin
date=" 00/00/0000 66:66"
date=strcompress(date, /remove_all)
endif else begin
date=date
endelse
min=strmid(date,13,2)
min=long(min)
hour=strmid(date,10,2)
hour=long(hour)
day=strmid(date,0,2)
day=long(day)
month=strmid(date,3,2)
month=long(month)
year=strmid(date,6,4)
year=long(year)

print, "Enter CME starting distance from the Sun in R_sun (for default value enter 0)"
R_0=0L
read, R_0

print, "Enter CME source position radial distance from the center of the disc in R_sun (0 < r ≤ 1; if unknown enter 0)"
r=0.0
read, r

print, "Enter CME sorce position central meridian distance in degrees (-90 ≤ CMD ≤ 90; if unknown enter 0)"
cmd=0L
read, cmd

print, "Enter CME width in degrees (0 < width ≤ 360; if unknown enter 0)"
width=0L
read, width

print, "Enter CME speed in km/s (v>0 for DBM; v>106 for FDFT; v>400 for CGeFT)"
v=0.0
read, v

print, "Enter associated flare soft X-ray intensity in *10^(-7)Wm^(-2) (f>5.3; if unknown enter 0)"
f=0.0
read, f

print, "Enter CME-CME interaction parameter (likeliness of CME-CME interaction: 1-no intzeraction; 2-interaction not likely; 3-interaction likely; 4-interaction highly likely, if unknown enter 0)"
i=0L
read, i

print, "Enter drag parameter gamma in 10^(-7) 1/km (for default value enter 0)"
gamma=0.0
read, gamma

print, "Enter solar wind speed in km/s (for default value enter 0)"
swspeed=0.0
read, swspeed

print, "Enter target distance from the Sun in AU (for default value-Earth enter 0 or 1)"
R_target=0.0
read, R_target

print, "Enter separation angle between the target and Earth in degrees (for default value-Earth enter 0)"
sep_angle=0.0
read, sep_angle

;CGeFT INPUT
cgeft_input=make_array(5,1)
cgeft_input(0)=v
cgeft_input(1)=r
case 1 of
(width gt 0) and (width lt 120):cgeft_input(2)=1
(width ge 120) and (width lt 360):cgeft_input(2)=2
(width eq 360):cgeft_input(2)=3
else:cgeft_input(2)=0
endcase
case 1 of
(f gt 0.) and (f lt 1.0E-5):cgeft_input(3)=1
(f ge 1.0E-5) and (f lt 1.0E-4):cgeft_input(3)=2
(f ge 1.0E-4): cgeft_input(3)=3
else: cgeft_input(3)=0
endcase
cgeft_input(4)=i
openw,lun,path + '\CGeFT\INPUT\cgeft_input.txt',/get_lun
printf,lun,cgeft_input
free_lun,lun

;FDFT INPUT
fdft_input=make_array(5,1)
fdft_input(0)=v
fdft_input(1)=r
fdft_input(2)=width
fdft_input(3)=f
fdft_input(4)=i
openw,lun,path + '\FDFT\INPUT\fdft_input.txt',/get_lun
printf,lun,fdft_input
free_lun,lun

;DBM INPUT
dbm_input=make_array(13,1)
dbm_input(0)=day
dbm_input(1)=month
dbm_input(2)=year
dbm_input(3)=hour
dbm_input(4)=min
dbm_input(5)=R_0
dbm_input(6)=v
dbm_input(7)=gamma
dbm_input(8)=swspeed
dbm_input(9)=R_target
dbm_input(10)=width
dbm_input(11)=cmd
dbm_input(12)=sep_angle
openw,lun,path + '\DBM\INPUT\dbm_input.txt',/get_lun
printf,lun,dbm_input
free_lun,lun

print, "Input files for DBM, CGeFT and FDFT have been created."
print, "You can now run dbm.pro, cgeft.pro and fdft.pro"
print, "The output of each model can be found in their corresponding folder under OUTPUT." 


end


